local _, AT = ...

local Settings = AT.Settings
local Utils = AT.Utils

local PROFILE_TYPE = AT.Enums.PROFILE_TYPE
local TRANSPARENCY_STATE = AT.Enums.TRANSPARENCY_STATE

local FrameBase = {}
FrameBase.padding = 7
FrameBase.dragable = false
FrameBase.allowedTransparencyStates = {}
FrameBase.id = nil
FrameBase.isVisible = false
FrameBase.transparencyStates = {
  [TRANSPARENCY_STATE.NO_MANA] = false,
  [TRANSPARENCY_STATE.NOT_IN_RANGE] = false
}
FrameBase.textColors = {
  headline = 'ffff00ff',
  unitName = 'ffff00ff',
  unitOrigin = 'ffff00ff',
  buffName = 'ffff00ff',
  buffCount = 'ffff00ff',
  buffExpTime = 'ffff00ff'
}

AT.FrameBase = FrameBase

function FrameBase:postConstruct(aId)
  self.id = aId
  self.transparencyStates = Utils:TableShallowCopy(self.transparencyStates)
end

function FrameBase:PostInit()
  if (self.dragable) then
    self.frame:RegisterForDrag('LeftButton')
    self.frame:SetScript('OnDragStart', function()
      if self.frame:IsMovable() then
        self.frame:StartMoving()
      end
    end)
    self.frame:SetScript('OnDragStop', function()
      self.frame:StopMovingOrSizing()
      self:SaveOffset()
      AT.FrameSettings:RefreshAllValues()
    end)

    if not Settings.frameDragWithModifierKey then
      self:ToggleDragging(true)
    end
  end

  Settings.OnProfileChanged:Add(self, self.OnProfileChanged, PROFILE_TYPE.GENERAL)

  self:UpdateSettings()
end

function FrameBase:OnProfileChanged(aProfileType)
  self:UpdateSettings()
end

function FrameBase:UpdateSettings(aOptions)
  aOptions = aOptions or {}

  local bdOptions = {}
  bdOptions.bgFile = 'Interface\\AddOns\\SpellstealMaster\\textures\\bg.tga'
  bdOptions.insets = { left = 4, right = 4, top = 4, bottom = 4 }
  if self:GetBorderEnabled() then
    bdOptions.tile = true
    bdOptions.edgeFile = 'Interface/Tooltips/UI-Tooltip-Border'
    bdOptions.tileSize = 16
    bdOptions.edgeSize = 16
  end
  self.frame:SetBackdrop(bdOptions)
  self.frame:SetBackdropColor(self:GetBackgroundColor())
  self.frame:SetBackdropBorderColor(self:GetBorderColor())

  for k, v in pairs(self.textColors) do
    self.textColors[k] = Utils:RgbToHexStr(Settings['color'..k:sub(1,1):upper()..k:sub(2)])
  end

  self.frame:SetScale(self:GetScale())

  self.frame:SetFrameStrata(self:GetStrata())

  if not aOptions.ignorePoint then
    local _, relativeTo = self.frame:GetPoint()
    self.frame:ClearAllPoints()
    self.frame:SetPoint(self:GetPivot(), relativeTo, self:GetAnchor(), self:GetOffset())
  end

  self:RefreshTransparency()
end

function FrameBase:ToggleTransparency(aState, aEnabled)
  if self.transparencyStates[aState] == aEnabled or not tContains(self.allowedTransparencyStates, aState) then
    return
  end
  self.transparencyStates[aState] = aEnabled
  self:RefreshTransparency()
end

function FrameBase:RefreshTransparency()
  local a
  if self.transparencyStates[TRANSPARENCY_STATE.NO_MANA] then
    a = Settings.alphaNoMana
  elseif self.transparencyStates[TRANSPARENCY_STATE.NOT_IN_RANGE] then
    a = Settings.alphaOutOfRange
  else
    a = 1
  end

  self.frame:SetAlpha(a)
end

function FrameBase:ToggleDragging(aEnabled)
  self.frame:SetMovable(aEnabled)
  if not aEnabled then
    self.frame:StopMovingOrSizing()
  end
end

function FrameBase:GetEnabled()
  return Settings[self.id..'Enabled']
end

function FrameBase:GetBackgroundColor()
  local c = Settings.colorBg
  return c.r, c.g, c.b, c.a
end

function FrameBase:GetBorderColor()
  local c = Settings.colorBorder
  return c.r, c.g, c.b, c.a
end

function FrameBase:GetBorderEnabled()
  return Settings[self.id..'ShowBorder']
end

function FrameBase:GetStrata()
  return Settings[self.id..'Strata']
end

function FrameBase:GetAnchor()
  return Settings[self.id..'Anchor']
end

function FrameBase:GetPivot()
  return Settings[self.id..'Pivot']
end

function FrameBase:GetOffset()
  return Utils:Round(Settings[self.id..'OffsetX']), Utils:Round(Settings[self.id..'OffsetY'])
end

function FrameBase:SaveOffset()
  local pivot, _, anchor, xOfs, yOfs = self.frame:GetPoint()
  Settings[self.id..'Pivot'] = pivot
  Settings[self.id..'Anchor'] = anchor
  Settings[self.id..'OffsetX'] = Utils:Round(xOfs)
  Settings[self.id..'OffsetY'] = Utils:Round(yOfs)
end

function FrameBase:GetScale()
  return Settings[self.id..'Scale']
end
